#include <LiquidCrystal.h>
#include <stdint.h>
#include <stdio.h> 
#include <dht.h>     //DHTLIB library
#include "myRegDefs.h"
#include "avr/interrupt.h"

const int rs = A5, en = A4, d4 = 2, d5 = 3, d6 = A2, d7 = A3;
int increment = 0;
int decrement = 0;
int location[] = {0,1};         //{menu page, cursor location}
float saveData[] = {0,0};
volatile uint8_t address = (0x000000);

const int AD_PIN = A1;
int buttonState;             // the current reading from the input pin
int lastButtonState = 0;   // the previous reading from the input pin
unsigned long lastDebounceTime = 0;  // the last time the output pin was toggled

LiquidCrystal lcd(rs,en,d4,d5,d6,d7);

byte arrow[8] = {
  0b00000,
  0b00100,
  0b00110,
  0b11111,
  0b00110,
  0b00100,
  0b00000,
  0b00000
};
//functions to save
//
//
void csLow(){
  PORTB_REG &= ~BIT2;
}

void csHigh(){
  PORTB_REG |= BIT2;
}

void clkTick(){
  PORTB_REG |= BIT5;
//  delay(1);
  PORTB_REG &= ~BIT5;
//  delay(1);
}

void sendByte(uint8_t data){
  uint8_t i;
  for(i=0; i<8; i++){
    if(data & 0x80){
      PORTB_REG |= BIT3;
    } else {
      PORTB_REG &= ~BIT3;
    }
    clkTick();
    data = data << 1;
  }  
}


uint8_t readByte(){
  uint8_t i;
  uint8_t r;
  r = 0;
  for(i=0; i<8; i++){
    r = r << 1;
    if(PINB_REG & BIT4){
      r |= 0x01;
    }
    clkTick();
  }
  return r;
}

void sendAddress(uint32_t address){
  uint8_t i;
  address = address << 8;
  for(i=0; i<24; i++){
    if(address & 0x80000000){
      PORTB_REG |= BIT3;
    } else {
      PORTB_REG &= ~BIT3;
    }
    clkTick();
    address = address << 1;
  } 
}

void writeData(uint32_t address, uint8_t data){
  csLow();
  delay(1);
  sendByte(WREN);
  csHigh();
  delay(3);
  csLow();
  delay(1);
  sendByte(WRITE);
  sendAddress(address);
  sendByte(data);
  csHigh();
  delay(3);
  csLow();
  delay(1);
  sendByte(WRDS);
  csHigh();
  delay(3); 
}

uint8_t readByteFromAddress(uint32_t address){
  uint8_t r = (0x00);
  csLow();
  delay(1);
  sendByte(READ);
  sendAddress(address);
  r = readByte();
  csHigh();
  delay(1);
  return r;
}

void chipErase(){
  csLow();
  delay(1);
  sendByte(WREN);
  csHigh();
  delay(3);
  csLow();
  delay(1);
  sendByte(ERASE);
  csHigh();
  delay(10000);
}

void save(int value){

  uint8_t data2;
  volatile uint8_t data1 = value;
  
  writeData(address, data1);
  delay(500);
  data2 = readByteFromAddress(address);
  delay(500);
  Serial.print(data2, HEX);
  
//  while(1){

//   data1 = readByteFromAddress(a);
//   Serial.println(data1, HEX);
//  }
address++;



}

void menuMaker(String line[]){
  lcd.clear();
  lcd.setCursor(3,0);
  lcd.print(line[0]);
  lcd.setCursor(1,1);
  lcd.print(line[1]);
  lcd.setCursor(1,2);
  lcd.print(line[2]);
  lcd.setCursor(1,3);
  lcd.print(line[3]);

  lcd.setCursor(0,location[1]);
  lcd.write(byte(0));
  
}
void menu(){
   lcd.clear();
switch(location[0]) {
  
    case(0):
    {
      String mainMenu[] = {"Main Menu","Sensors","Memory","Settings"};
      menuMaker(mainMenu);
      
      increment = 10;
      decrement = 0;
    break;
    }
    case(10):
    {
      String sensorsMenu[] = {"Sensors 1","Voltmeter","Scale","More..."};
      menuMaker(sensorsMenu);
      
      increment = 1;
      decrement = 10;
    break;
    }
    case 11:
    {
      location[1] = 3;
      String voltmeterMenu[] = {"Voltmeter","Volts: "," ","Save"};
      menuMaker(voltmeterMenu);

      voltmeter();
      while(analogRead(AD_PIN) == 0){
          voltmeter();
      }

      increment = 10;
      decrement = 1;
    break;
    }
    case 12:
    {
      location[1] = 3;
//      weight = 5;                                                        //add weight function call
      String scaleMenu[] = {"Scale", "Weight: ","","Save"};
      menuMaker(scaleMenu);

      increment = 0;
      decrement = 2;
    break;
    }
    case 13:
    {                                                                 
      String sensorsMenu2[] = {"Sensors 2", "Distance","Temp/humidity","More..."};
      menuMaker(sensorsMenu2);

      increment = 1;
      decrement = 3;
    break;
    }
    case 14:
    {
      location[1] = 3;
      
      String distanceMenu[] = {"Distance(cm)", ""," ","Save"};
      menuMaker(distanceMenu);

      distance();
      delay(50);
      while(analogRead(AD_PIN) == 0){
        distance();
      }
      increment = 10;
      decrement = 1;

      
    break;
    }
    case 15:
    {
      location[1] = 3;
      String tempHumidityMenu[] = {"Temp/Humidity", "Temp(F): ","Humidity(%): ","Save"};
      menuMaker(tempHumidityMenu);
      
        temphum();
        while(analogRead(AD_PIN) == 0){
          temphum();
        }
 
      increment = 10;
      decrement = 2;                                            
    break;
    }
    case 18:
    {
      location[1] = 3;
//    temp = 10;                                                      //add thermometer (water proof) call
      String thermometerMenu[] = {"Temperature", "Temp(c):  ","","Save"};     
      menuMaker(thermometerMenu);

      increment = 0; 
      decrement = 2;
    break;
    }
    case 16:
    {
      String sensorsMenu3[] = {"Sensors 3", "Beam Break","Temperature","PH"};
      menuMaker(sensorsMenu3);

      increment = 1;
      decrement = 3;
    break;
    }
    case 17:
    {
      location[1] = 3;
//    disp = 10;                                                                //add diff time call
//    velocity = 10;                                                             //add velocity call
      String beambreakMenu[] = {"Beam Break", "time: ","Velocity: ","Save"};
      menuMaker(beambreakMenu);

      increment = 0;
      decrement = 1;
    break;
    }
    case 19:
    {
      location[1] = 3;
      String phMenu[] = {"PH Sensor", "PH: "," ","Save"};
      menuMaker(phMenu);

      ph();
      while(analogRead(AD_PIN) == 0){
          ph();
      }
      

      increment = 10;
      decrement = 3;
    break;
    }
    case 20:
    {
      String memoryMenu[] = {"Memory","Export Data","Clear All","Sensors Menu"};
      menuMaker(memoryMenu);

      increment = 1;
      decrement = 20;
    break;
    }
    case 21:
    {
      //export data
      location[0] = 20;
      menu();
      break;
    }
    case 22:
    {
      lcd.setCursor(2,2);
      lcd.print("Deleting...");
      chipErase();
      address = (0x000000);
      location[0] = 20;
      menu();
      break;
    }
    case 23:
    {
      location[0] = 10;
      menu();
      break;
    }
    case 30:
    {
      String settingsMenu[] = {"Settings","sleep","Authors","product info"};
      menuMaker(settingsMenu);

      increment = 1;
      decrement = 30;
    break;
    }
   case 31:
   {
      lcd.clear();
      delay(500);
      while(analogRead(AD_PIN) == 0){
        }
      location[0] = 0;
      menu();
   break;
   }
   case 32:
   {
      String authorInfo[] = {"Authors","Roxy","THE","CAT"};
      menuMaker(authorInfo);
      increment = 0;
      decrement = 2;
   break;
   }
   case 33:
   {
      String productInfo[] = {"Product Info","multi-measure","Memory: 16mb","Micro:Atmega328"};
      menuMaker(productInfo);
      increment = 0;
      decrement = 3;

   break;
   }
   case 41:
   {
    lcd.setCursor(3,2);
    lcd.print(" SAVING...");
    uint8_t vinF;

    vinF = toINT(saveData[0]);

    save(vinF);
    Serial.println(" Volts");
    location[0] = 11;
    menu();
    break;     
   }
   case 44:
   {
    lcd.setCursor(3,2);
    lcd.print(" SAVING...");
    uint8_t distanceF;

    distanceF = toINT(saveData[0]);

    save(distanceF);
    Serial.println(" cm");
    location[0] = 14;
    menu();
    break;    
   }
   case 45:
   {
    lcd.setCursor(3,2);
    lcd.print(" SAVING...");
    int tempF;
    int humF;
    
    tempF = toINT(saveData[0]);
    humF = toINT(saveData[1]);

    //send to save
    save(tempF);
    save(humF);
    location[0] = 15;
    menu();
    break;
   }
   case 49:
   {
    lcd.setCursor(3,2);
    lcd.print(" SAVING...");
    int phF;
    
    phF = toINT(saveData[0]);

    save(phF);
    Serial.println(" ph");

    location[0] = 19;
    menu();
    break;     
   } 
 } 
}

int debounce(int adValue){
  unsigned long debounceDelay = 70; 
  
  if ((adValue > (lastButtonState + 20)) || (adValue < lastButtonState - 20)) {
    // reset the debouncing timer
    lastDebounceTime = millis();
  }
  
  if ((millis() - lastDebounceTime) > debounceDelay) {
      if ((adValue > (buttonState + 20) || (adValue < (buttonState - 20)))) {
        buttonState = adValue;
        lastButtonState = adValue;
        return buttonState;
      }
  }
    
  lastButtonState = adValue;
  return 0;
}

void buttonFromValue(int adValue) {

  if (adValue > 260 && adValue < 281) {   //red   back now up
    
    location[1]--;
    if (location[1] == 0){
      location[1] = 3;
    }
  }   

  if (adValue > 600 && adValue < 700) {  //white  go up now forward

   location[0] = location[0] + (increment * location[1]);
   location[1] = 1;

  }

  if (adValue > 409 && adValue < 413) { //purple go down now back
    
    location[0] = location[0] - decrement;
    location[1] = 1;
    
  }

  if (adValue > 1020) { // blue go forward now down
   
   location[1]++;
   if (location[1] == 4){
      location[1] = 1;
   }
   
  }
}

void distance(){
  int trigPin = 6;
  int echoPin = 5;
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);
  float duration, distance;

  digitalWrite(trigPin, LOW); 
  delayMicroseconds(2);
 
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);
  
  duration = pulseIn(echoPin, HIGH);
  distance = (duration / 2) * 0.0344;
  lcd.setCursor(3,1);
  if (distance >= 400 || distance <= 2){
    lcd.print("Out of Range");
  }
  else{
    saveData[0] = distance;
    lcd.print(distance);
    lcd.print("       ");
  }
  delay(500);
}

void temphum(){
  dht DHT;
  int DHT11_PIN = 6;
  
  int chk = DHT.read11(DHT11_PIN);
  float tempC = DHT.temperature;
  float tempF = (tempC * (1.8)) + 32; 
  float hum = DHT.humidity; 
  if(hum > 10  && tempF < 100){
  saveData[0] = tempC;
  saveData[1] = hum;
  
  lcd.setCursor(10,1);
  lcd.print(tempF);

  lcd.setCursor(14,2);
  lcd.print(hum);

  delay(250); 
  }
}

void ph(){
unsigned long int avgValue;  //Store the avg value of the sensor feedback
int SensorPin = A0;
float b;
int buf[10],temp;
  
 for(int i=0;i<10;i++)       //Get 10 sample value from the sensor
  { 
    buf[i]=analogRead(SensorPin) * 10;
    delay(10);
  }
  for(int i=0;i<9;i++)        //sort the analog from small to large
  {
    for(int j=i+1;j<10;j++)
    {
      if(buf[i]>buf[j])
      {
        temp=buf[i];
        buf[i]=buf[j];
        buf[j]=temp;
      }
    }
  }
  avgValue=0;
  for(int i=2;i<8;i++)                      //take the average value of 6 center sample
    avgValue+=buf[i];
  float phValue=(float)avgValue*5.0/1024/6; //convert the analog into millivolt
  phValue=3.5*phValue;                      //convert the millivolt into pH value
  
  saveData[0] = phValue;
  lcd.setCursor(10,1);
  lcd.print(phValue); 
  
}

int toINT(float value){
  int newValue = (int)value;
  return newValue;
  
}
void voltmeter(){
  int volt;
  float voltage;
  float vin = 0.00;
  float R1 = 100000;
  float R2 = 9600;

  volt = analogRead(A0);
  delay(100);
  voltage = volt * (5.0 / 1024);
  vin = voltage * (R1 / (R1+R2)) * 13;

  saveData[0] = vin;
  lcd.setCursor(10,1);
  lcd.print(vin);
  delay(1000);
}

int main(){
  
  init();
  Serial.begin(9600);
  csHigh();
  DDRB_REG = (0x2C);
  DDRB_REG |= BIT1;   //Turn on chip
  PORTB_REG |= BIT1;
  delay(1000);
  lcd.begin(16,4);
  lcd.createChar(0, arrow);

  menu();
  delay(10);
  while(1){
    
    int pinValue = analogRead(AD_PIN);
    int buttonval = debounce(pinValue);

    if(buttonval != 0){
      buttonFromValue(pinValue); 
      menu(); 
    } 
  }

  return;
}
